diff --git a/core/core.services.yml b/core/core.services.yml
index cb832707bbfeebb806f7a551d013c45f09662088..24ef51ea02f54da9386ee226daa7764928f19582 100644
--- a/core/core.services.yml
+++ b/core/core.services.yml
@@ -174,7 +174,7 @@ services:
     arguments: ['@container.namespaces']
   plugin.manager.menu.local_action:
     class: Drupal\Core\Menu\LocalActionManager
-    arguments: ['@container.namespaces', '@controller_resolver', '@request', '@module_handler', '@cache.cache', '@language_manager']
+    arguments: ['@controller_resolver', '@request', '@router.route_provider', '@module_handler', '@cache.cache', '@language_manager', '@access_manager']
   plugin.manager.menu.local_task:
     class: Drupal\Core\Menu\LocalTaskManager
     arguments: ['@controller_resolver', '@request', '@router.route_provider', '@module_handler', '@cache.cache', '@language_manager', '@access_manager']
diff --git a/core/includes/menu.inc b/core/includes/menu.inc
index 3c29df6caf007cdf4c959aff96920c9ef9462fff..c8fddcd84f09ed4665b1de4e32d4f99ade6f5d73 100644
--- a/core/includes/menu.inc
+++ b/core/includes/menu.inc
@@ -1763,12 +1763,21 @@ function theme_menu_local_action($variables) {
   $link += array(
     'href' => '',
     'localized_options' => array(),
+    'route_parameters' => array(),
   );
   $link['localized_options']['attributes']['class'][] = 'button';
   $link['localized_options']['attributes']['class'][] = 'button-action';
 
   $output = '<li>';
-  $output .= l($link['title'], $link['href'], $link['localized_options']);
+  // @todo Remove this check and the call to l() when all pages are converted to
+  //   routes.
+  // @todo Figure out how to support local actions without a href properly.
+  if ($link['href'] === '' && !empty($link['route_name'])) {
+    $output .= Drupal::l($link['title'], $link['route_name'], $link['route_parameters'], $link['localized_options']);
+  }
+  else {
+    $output .= l($link['title'], $link['href'], $link['localized_options']);
+  }
   $output .= "</li>";
 
   return $output;
@@ -2316,21 +2325,9 @@ function menu_secondary_local_tasks() {
  */
 function menu_get_local_actions() {
   $links = menu_local_tasks();
-  $router_item = menu_get_item();
+  $route_name = Drupal::request()->attributes->get(RouteObjectInterface::ROUTE_NAME);
   $manager = \Drupal::service('plugin.manager.menu.local_action');
-  $local_actions = $manager->getActionsForRoute($router_item['route_name']);
-  foreach ($local_actions as $plugin) {
-    $route_path = $manager->getPath($plugin);
-    $action_router_item = menu_get_item($route_path);
-    $links['actions'][$route_path] = array(
-      '#theme' => 'menu_local_action',
-      '#link' => array(
-        'title' => $manager->getTitle($plugin),
-        'href' => $route_path,
-      ),
-      '#access' => $action_router_item['access'],
-    );
-  }
+  $links['actions'] += $manager->getActionsForRoute($route_name);
   return $links['actions'];
 }
 
diff --git a/core/lib/Drupal/Core/Annotation/Menu/LocalAction.php b/core/lib/Drupal/Core/Annotation/Menu/LocalAction.php
deleted file mode 100644
index 3c4cdb333357c2f2963f0a3da88bf97cbbdb9069..0000000000000000000000000000000000000000
--- a/core/lib/Drupal/Core/Annotation/Menu/LocalAction.php
+++ /dev/null
@@ -1,49 +0,0 @@
-<?php
-
-/**
- * @file
- * Contains \Drupal\Core\Annotation\LocalAction.
- */
-
-namespace Drupal\Core\Annotation\Menu;
-
-use Drupal\Component\Annotation\Plugin;
-
-/**
- * Defines a LocalAction type Plugin annotation object.
- *
- * @Annotation
- */
-class LocalAction extends Plugin {
-
-  /**
-   * The ID.
-   *
-   * @var string
-   */
-  public $id;
-
-  /**
-   * A static title for the local action.
-   *
-   * @ingroup plugin_translatable
-   *
-   * @var \Drupal\Core\Annotation\Translation
-   */
-  public $title;
-
-  /**
-   * The route name.
-   *
-   * @var string
-   */
-  public $route_name;
-
-  /**
-   * An array of route names where this action appears.
-   *
-   * @var array (optional)
-   */
-  public $appears_on = array();
-
-}
diff --git a/core/lib/Drupal/Core/Menu/LocalActionBase.php b/core/lib/Drupal/Core/Menu/LocalActionBase.php
deleted file mode 100644
index c99fd78e4702b2e1ee811da2c02a0555357890f3..0000000000000000000000000000000000000000
--- a/core/lib/Drupal/Core/Menu/LocalActionBase.php
+++ /dev/null
@@ -1,104 +0,0 @@
-<?php
-
-/**
- * @file
- * Contains \Drupal\Core\Menu\LocalActionBase.
- */
-
-namespace Drupal\Core\Menu;
-
-use Drupal\Core\Menu\LocalActionInterface;
-use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
-use Drupal\Core\Plugin\PluginBase;
-use Drupal\Core\StringTranslation\Translator\TranslatorInterface;
-use Symfony\Component\DependencyInjection\ContainerInterface;
-use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
-
-/**
- * Provides defaults and base methods for menu local action plugins.
- *
- * @todo This class needs more documentation and/or @see references.
- */
-abstract class LocalActionBase extends PluginBase implements LocalActionInterface, ContainerFactoryPluginInterface {
-
-  /**
-   * String translation object.
-   *
-   * @var \Drupal\Core\StringTranslation\Translator\TranslatorInterface
-   */
-  protected $t;
-
-  /**
-   * URL generator object.
-   *
-   * @var \Symfony\Component\Routing\Generator\UrlGeneratorInterface
-   */
-  protected $generator;
-
-  /**
-   * Constructs a LocalActionBase object.
-   *
-   * @param \Drupal\Core\StringTranslation\Translator\TranslatorInterface $string_translation
-   *   A translator object for use by subclasses generating localized titles.
-   * @param \Symfony\Component\Routing\Generator\UrlGeneratorInterface $generator
-   *   A URL generator object used to get the path from the route.
-   * @param array $configuration
-   *   A configuration array containing information about the plugin instance.
-   * @param string $plugin_id
-   *   The plugin_id for the plugin instance.
-   * @param array $plugin_definition
-   *   The plugin implementation definition.
-   */
-  public function __construct(TranslatorInterface $string_translation,  UrlGeneratorInterface $generator, array $configuration, $plugin_id, array $plugin_definition) {
-    parent::__construct($configuration, $plugin_id, $plugin_definition);
-
-    $this->generator = $generator;
-    // This is available for subclasses that need to translate a dynamic title.
-    $this->t = $string_translation;
-  }
-
-  /**
-   * {@inheritdoc}
-   */
-  public static function create(ContainerInterface $container, array $configuration, $plugin_id, array $plugin_definition) {
-    return new static(
-      $container->get('string_translation'),
-      $container->get('url_generator'),
-      $configuration,
-      $plugin_id,
-      $plugin_definition
-    );
-  }
-
-  /**
-   * {@inheritdoc}
-   */
-  public function getRouteName() {
-    return $this->pluginDefinition['route_name'];
-  }
-
-  /**
-   * {@inheritdoc}
-   */
-  public function getTitle() {
-    // Subclasses may pull in the request or specific attributes as parameters.
-    return $this->pluginDefinition['title'];
-  }
-
-  /**
-   * {@inheritdoc}
-   */
-  public function getPath() {
-    // Subclasses may set a request into the generator or use any desired method
-    // to generate the path.
-    // @todo - use the new method from https://drupal.org/node/2031353
-    $path = $this->generator->generate($this->getRouteName());
-    // In order to get the Drupal path the base URL has to be stripped off.
-    $base_url = $this->generator->getContext()->getBaseUrl();
-    if (!empty($base_url) && strpos($path, $base_url) === 0) {
-      $path = substr($path, strlen($base_url));
-    }
-    return trim($path, '/');
-  }
-
-}
diff --git a/core/lib/Drupal/Core/Menu/LocalActionDefault.php b/core/lib/Drupal/Core/Menu/LocalActionDefault.php
new file mode 100644
index 0000000000000000000000000000000000000000..be6e6abf1ee86ebe54f8d2b3692d96081500e1ce
--- /dev/null
+++ b/core/lib/Drupal/Core/Menu/LocalActionDefault.php
@@ -0,0 +1,119 @@
+<?php
+
+/**
+ * @file
+ * Contains \Drupal\Core\Menu\LocalActionDefault.
+ */
+
+namespace Drupal\Core\Menu;
+
+use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
+use Drupal\Core\Plugin\PluginBase;
+use Drupal\Core\Routing\RouteProviderInterface;
+use Symfony\Component\DependencyInjection\ContainerInterface;
+use Symfony\Component\HttpFoundation\Request;
+
+/**
+ * Provides a default implementation for local action plugins.
+ */
+class LocalActionDefault extends PluginBase implements LocalActionInterface, ContainerFactoryPluginInterface {
+
+  /**
+   * The route provider to load routes by name.
+   *
+   * @var \Drupal\Core\Routing\RouteProviderInterface
+   */
+   protected $routeProvider;
+
+  /**
+   * Constructs a LocalActionDefault object.
+   *
+   * @param array $configuration
+   *   A configuration array containing information about the plugin instance.
+   * @param string $plugin_id
+   *   The plugin_id for the plugin instance.
+   * @param array $plugin_definition
+   *   The plugin implementation definition.
+   * @param \Drupal\Core\Routing\RouteProviderInterface $route_provider
+   *   The route provider to load routes by name.
+   */
+  public function __construct(array $configuration, $plugin_id, array $plugin_definition, RouteProviderInterface $route_provider) {
+    parent::__construct($configuration, $plugin_id, $plugin_definition);
+
+    $this->routeProvider = $route_provider;
+  }
+
+  /**
+   * {@inheritdoc}
+   */
+  public static function create(ContainerInterface $container, array $configuration, $plugin_id, array $plugin_definition) {
+    return new static(
+      $configuration,
+      $plugin_id,
+      $plugin_definition,
+      $container->get('router.route_provider')
+    );
+  }
+
+  /**
+   * {@inheritdoc}
+   */
+  public function getRouteName() {
+    return $this->pluginDefinition['route_name'];
+  }
+
+  /**
+   * {@inheritdoc}
+   */
+  public function getTitle() {
+    // Subclasses may pull in the request or specific attributes as parameters.
+    return $this->t($this->pluginDefinition['title']);
+  }
+
+  /**
+   * {@inheritdoc}
+   */
+  public function getWeight() {
+    return $this->pluginDefinition['weight'];
+  }
+
+  /**
+   * {@inheritdoc}
+   */
+  public function getRouteParameters(Request $request) {
+    $parameters = isset($this->pluginDefinition['route_parameters']) ? $this->pluginDefinition['route_parameters'] : array();
+    $route = $this->routeProvider->getRouteByName($this->getRouteName());
+    $variables = $route->compile()->getVariables();
+
+    // Normally the \Drupal\Core\ParamConverter\ParamConverterManager has
+    // processed the Request attributes, and in that case the _raw_variables
+    // attribute holds the original path strings keyed to the corresponding
+    // slugs in the path patterns. For example, if the route's path pattern is
+    // /filter/tips/{filter_format} and the path is /filter/tips/plain_text then
+    // $raw_variables->get('filter_format') == 'plain_text'.
+    $raw_variables = $request->attributes->get('_raw_variables');
+
+    foreach ($variables as $name) {
+      if (isset($parameters[$name])) {
+        continue;
+      }
+
+      if ($raw_variables && $raw_variables->has($name)) {
+        $parameters[$name] = $raw_variables->get($name);
+      }
+      elseif ($request->attributes->has($name)) {
+        $parameters[$name] = $request->attributes->get($name);
+      }
+    }
+    // The UrlGenerator will throw an exception if expected parameters are
+    // missing. This method should be overridden if that is possible.
+    return $parameters;
+  }
+
+  /**
+   * {@inheritdoc}
+   */
+  public function getOptions(Request $request) {
+    return (array) $this->pluginDefinition['options'];
+  }
+}
diff --git a/core/lib/Drupal/Core/Menu/LocalActionInterface.php b/core/lib/Drupal/Core/Menu/LocalActionInterface.php
index 65287b6557f87856608092d1b821068bfa7f7ba6..152b3c62da0b8683cc08a119a6acfa1863714ba7 100644
--- a/core/lib/Drupal/Core/Menu/LocalActionInterface.php
+++ b/core/lib/Drupal/Core/Menu/LocalActionInterface.php
@@ -7,6 +7,8 @@
 
 namespace Drupal\Core\Menu;
 
+use Symfony\Component\HttpFoundation\Request;
+
 /**
  * Defines an interface for menu local actions.
  */
@@ -20,6 +22,39 @@ interface LocalActionInterface {
    */
   public function getRouteName();
 
+  /**
+   * Returns the route parameters needed to render a link for the local action.
+   *
+   * @param \Symfony\Component\HttpFoundation\Request $request
+   *   The HttpRequest object representing the current request.
+   *
+   * @return array
+   *   An array of parameter names and values.
+   *
+   * @see \Drupal\Core\Utility\LinkGeneratorInterface::generate()
+   */
+  public function getRouteParameters(Request $request);
+
+   /**
+    * Returns the weight for the local action.
+    *
+    * @return int
+    */
+   public function getWeight();
+
+  /**
+   * Returns options for rendering a link for the local action.
+   *
+   * @param \Symfony\Component\HttpFoundation\Request $request
+   *   The HTTP request object representing the current request.
+   *
+   * @return array
+   *   An associative array of options.
+   *
+   * @see \Drupal\Core\Utility\LinkGeneratorInterface::generate()
+   */
+  public function getOptions(Request $request);
+
   /**
    * Returns the localized title to be shown for this action.
    *
@@ -33,17 +68,6 @@ public function getRouteName();
    */
   public function getTitle();
 
-  /**
-   * Return an internal Drupal path to use when linking to the action.
-   *
-   * Subclasses may add arguments for request attributes which will then be
-   * automatically supplied by the controller resolver.
-   *
-   * @return string
-   *   The path to use when linking to the action.
-   *
-   * @see \Drupal\Core\Menu\LocalActionManager::getPath()
-   */
-  public function getPath();
+
 
 }
diff --git a/core/lib/Drupal/Core/Menu/Plugin/Derivative/StaticLocalActionDeriver.php b/core/lib/Drupal/Core/Menu/Plugin/Derivative/StaticLocalActionDeriver.php
deleted file mode 100644
index e9ef0ee07de34d63e4b13867a3bea57061dc1a67..0000000000000000000000000000000000000000
--- a/core/lib/Drupal/Core/Menu/Plugin/Derivative/StaticLocalActionDeriver.php
+++ /dev/null
@@ -1,124 +0,0 @@
-<?php
-
-/**
- * @file
- * Contains \Drupal\Core\Menu\Plugin\Derivative\StaticLocalActionDeriver.
- */
-
-namespace Drupal\Core\Menu\Plugin\Derivative;
-
-use Drupal\Core\Extension\ModuleHandlerInterface;
-use Drupal\Core\Plugin\Discovery\ContainerDerivativeInterface;
-use Drupal\Component\Plugin\Exception\PluginException;
-use Drupal\Core\StringTranslation\TranslationInterface;
-use Drupal\Component\Utility\String;
-use Drupal\Component\Discovery\YamlDiscovery;
-use Symfony\Component\DependencyInjection\ContainerInterface;
-
-/**
- * Provides plugin derivatives for local actions provided in YAML files.
- */
-class StaticLocalActionDeriver implements ContainerDerivativeInterface {
-
-  /**
-   * List of derivative definitions.
-   *
-   * @var array
-   */
-  protected $derivatives = array();
-
-  /**
-   * The base plugin ID.
-   *
-   * @var string
-   */
-  protected $basePluginId;
-
-  /**
-   * The module handler.
-   *
-   * @var \Drupal\Core\Extension\ModuleHandlerInterface
-   */
-  protected $moduleHandler;
-
-  /**
-   * The translation manager.
-   *
-   * @var \Drupal\Core\StringTranslation\TranslationInterface
-   */
-  protected $translationManager;
-
-  /**
-   * {@inheritdoc}
-   */
-  public static function create(ContainerInterface $container, $base_plugin_id) {
-    return new static(
-      $base_plugin_id,
-      $container->get('module_handler'),
-      $container->get('string_translation')
-    );
-  }
-
-  /**
-   * Constructs a StaticLocalActionDeriver object.
-   *
-   * @param string $base_plugin_id
-   *   The base plugin ID.
-   * @param\Drupal\Core\Extension\ModuleHandlerInterface $module_handler
-   *   The module handler.
-   * @param \Drupal\Core\StringTranslation\TranslationInterface translation_manager
-   *   The translation manager.
-   */
-  public function __construct($base_plugin_id, ModuleHandlerInterface $module_handler, TranslationInterface $translation_manager) {
-    $this->basePluginId = $base_plugin_id;
-    $this->moduleHandler = $module_handler;
-    $this->translationManager = $translation_manager;
-  }
-
-  /**
-   * {@inheritdoc}
-   */
-  public function getDerivativeDefinition($derivative_id, array $base_plugin_definition) {
-    if (!empty($this->derivatives) && !empty($this->derivatives[$derivative_id])) {
-      return $this->derivatives[$derivative_id];
-    }
-    $this->getDerivativeDefinitions($base_plugin_definition);
-    return $this->derivatives[$derivative_id];
-  }
-
-  /**
-   * {@inheritdoc}
-   */
-  public function getDerivativeDefinitions(array $base_plugin_definition) {
-    $yaml_discovery = new YamlDiscovery('local_actions', $this->moduleHandler->getModuleDirectories());
-    $required_keys = array('title' => 1, 'route_name' => 1, 'appears_on' => 1);
-
-    foreach ($yaml_discovery->findAll() as $module => $local_actions) {
-      if (!empty($local_actions)) {
-        foreach ($local_actions as $name => $info) {
-          if ($missing_keys = array_diff_key($required_keys, array_intersect_key($info, $required_keys))) {
-            throw new PluginException(String::format('Static local action @name is missing @keys', array('@name' => $name, '@keys' => implode(', ', array_keys($missing_keys)))));
-          }
-
-          $info += array('provider' => $module);
-          // Make sure 'appears_on' is an array.
-          $info['appears_on'] = (array) $info['appears_on'];
-          $info['title'] = $this->t($info['title']);
-          $this->derivatives[$name] = $info + $base_plugin_definition;
-        }
-      }
-    }
-
-    return $this->derivatives;
-  }
-
-  /**
-   * Translates a string to the current language or to a given language.
-   *
-   * See the t() documentation for details.
-   */
-  protected function t($string, array $args = array(), array $options = array()) {
-    return $this->translationManager->translate($string, $args, $options);
-  }
-
-}
diff --git a/core/lib/Drupal/Core/Menu/Plugin/Menu/LocalAction/StaticLocalAction.php b/core/lib/Drupal/Core/Menu/Plugin/Menu/LocalAction/StaticLocalAction.php
deleted file mode 100644
index b0e0e9d083b841c77fc2b86209692bdf58b9942b..0000000000000000000000000000000000000000
--- a/core/lib/Drupal/Core/Menu/Plugin/Menu/LocalAction/StaticLocalAction.php
+++ /dev/null
@@ -1,21 +0,0 @@
-<?php
-
-/**
- * @file
- * Contains \Drupal\Core\Annotation\StaticLocalAction.
- */
-
-namespace Drupal\Core\Menu\Plugin\Menu\LocalAction;
-
-use Drupal\Core\Annotation\Menu\LocalAction;
-use Drupal\Core\Menu\LocalActionBase;
-
-/**
- * @LocalAction(
- *   id = "local_action_static",
- *   derivative = "Drupal\Core\Menu\Plugin\Derivative\StaticLocalActionDeriver"
- * )
- */
-class StaticLocalAction extends LocalActionBase {
-
-}
diff --git a/core/modules/config/tests/config_test/config_test.local_actions.yml b/core/modules/config/tests/config_test/config_test.local_actions.yml
new file mode 100644
index 0000000000000000000000000000000000000000..8e1e4a4c5dfc77d7185e76bcd5ffe0e13812eb4b
--- /dev/null
+++ b/core/modules/config/tests/config_test/config_test.local_actions.yml
@@ -0,0 +1,5 @@
+config_test_entity_add_local_action:
+  route_name: config_test.entity_add
+  title: 'Add test configuration'
+  appears_on:
+    - config_test.list_page
diff --git a/core/modules/config/tests/config_test/config_test.module b/core/modules/config/tests/config_test/config_test.module
index b81561027445da74cd5aa1fd9007d5c8b3bd95f6..48048ad78c57d5112dc65ef915af007d8259591d 100644
--- a/core/modules/config/tests/config_test/config_test.module
+++ b/core/modules/config/tests/config_test/config_test.module
@@ -18,11 +18,6 @@ function config_test_menu() {
     'title' => 'Test configuration',
     'route_name' => 'config_test.list_page',
   );
-  $items['admin/structure/config_test/add'] = array(
-    'title' => 'Add test configuration',
-    'route_name' => 'config_test.entity_add',
-    'type' => MENU_SIBLING_LOCAL_TASK,
-  );
   $items['admin/structure/config_test/manage/%config_test'] = array(
     'route_name' => 'config_test.entity',
   );
diff --git a/core/modules/config/tests/config_test/lib/Drupal/config_test/Plugin/Menu/LocalAction/AddConfigTestEntityLocalAction.php b/core/modules/config/tests/config_test/lib/Drupal/config_test/Plugin/Menu/LocalAction/AddConfigTestEntityLocalAction.php
deleted file mode 100644
index 1f31d46008c9fc344f3d3387a8f901ba2d2f4209..0000000000000000000000000000000000000000
--- a/core/modules/config/tests/config_test/lib/Drupal/config_test/Plugin/Menu/LocalAction/AddConfigTestEntityLocalAction.php
+++ /dev/null
@@ -1,24 +0,0 @@
-<?php
-
-/**
- * @file
- * Contains \Drupal\config_test\Plugin\Menu\AddConfigTestEntityLocalAction.
- */
-
-namespace Drupal\config_test\Plugin\Menu\LocalAction;
-
-use Drupal\Core\Annotation\Translation;
-use Drupal\Core\Menu\LocalActionBase;
-use Drupal\Core\Annotation\Menu\LocalAction;
-
-/**
- * @LocalAction(
- *   id = "config_test_entity_add_local_action",
- *   route_name = "config_test.entity_add",
- *   title = @Translation("Add test configuration"),
- *   appears_on = {"config_test.list_page"}
- * )
- */
-class AddConfigTestEntityLocalAction extends LocalActionBase {
-
-}
diff --git a/core/modules/filter/filter.local_actions.yml b/core/modules/filter/filter.local_actions.yml
new file mode 100644
index 0000000000000000000000000000000000000000..0ddced4deec56a7f9c05c002795b087f353d0590
--- /dev/null
+++ b/core/modules/filter/filter.local_actions.yml
@@ -0,0 +1,5 @@
+filter_format_add_local_action:
+  route_name: filter.format_add
+  title: 'Add text format'
+  appears_on:
+    - filter.admin_overview
diff --git a/core/modules/filter/filter.module b/core/modules/filter/filter.module
index ea9896584313477bcb0fdc8fe613402d0c5d8e9b..49cde200308e8adbeb259b8c67d9cc9824b08d87 100644
--- a/core/modules/filter/filter.module
+++ b/core/modules/filter/filter.module
@@ -140,10 +140,6 @@ function filter_menu() {
     'title' => 'List',
     'type' => MENU_DEFAULT_LOCAL_TASK,
   );
-  $items['admin/config/content/formats/add'] = array(
-    'route_name' => 'filter.format_add',
-    'type' => MENU_SIBLING_LOCAL_TASK,
-  );
   $items['admin/config/content/formats/manage/%'] = array(
     'title callback' => 'entity_page_label',
     'title arguments' => array(5),
diff --git a/core/modules/filter/lib/Drupal/filter/Plugin/Menu/LocalAction/AddFilterFormatLocalAction.php b/core/modules/filter/lib/Drupal/filter/Plugin/Menu/LocalAction/AddFilterFormatLocalAction.php
deleted file mode 100644
index b75c10217c46229076cce58fe327e6d756f220b4..0000000000000000000000000000000000000000
--- a/core/modules/filter/lib/Drupal/filter/Plugin/Menu/LocalAction/AddFilterFormatLocalAction.php
+++ /dev/null
@@ -1,24 +0,0 @@
-<?php
-
-/**
- * @file
- * Contains \Drupal\filter\Plugin\Menu\AddFilterFormatLocalAction.
- */
-
-namespace Drupal\filter\Plugin\Menu\LocalAction;
-
-use Drupal\Core\Annotation\Translation;
-use Drupal\Core\Menu\LocalActionBase;
-use Drupal\Core\Annotation\Menu\LocalAction;
-
-/**
- * @LocalAction(
- *   id = "filter_format_add_local_action",
- *   route_name = "filter.format_add",
- *   title = @Translation("Add text format"),
- *   appears_on = {"filter.admin_overview"}
- * )
- */
-class AddFilterFormatLocalAction extends LocalActionBase {
-
-}
diff --git a/core/modules/image/image.local_actions.yml b/core/modules/image/image.local_actions.yml
new file mode 100644
index 0000000000000000000000000000000000000000..134f3a83e5ac8604ea0d71f3cafa96ff0c6ae893
--- /dev/null
+++ b/core/modules/image/image.local_actions.yml
@@ -0,0 +1,5 @@
+image_style_add_action:
+  route_name: image.style_add
+  title: 'Add image style'
+  appears_on:
+    - image.style_list
diff --git a/core/modules/image/image.module b/core/modules/image/image.module
index 8947ab0c38a63b44faee79268fdf1c1e26d09b1c..137bf668e0d166a561ac4d3dd01e41e60d5b9128 100644
--- a/core/modules/image/image.module
+++ b/core/modules/image/image.module
@@ -93,11 +93,6 @@ function image_menu() {
     'description' => 'List the current image styles on the site.',
     'type' => MENU_DEFAULT_LOCAL_TASK,
   );
-  $items['admin/config/media/image-styles/add'] = array(
-    'route_name' => 'image.style_add',
-    'type' => MENU_SIBLING_LOCAL_TASK,
-    'weight' => 1,
-  );
   $items['admin/config/media/image-styles/manage/%image_style'] = array(
     'title' => 'Edit style',
     'description' => 'Configure an image style.',
diff --git a/core/modules/image/lib/Drupal/image/Plugin/Menu/LocalAction/ImageStyleAddLocalAction.php b/core/modules/image/lib/Drupal/image/Plugin/Menu/LocalAction/ImageStyleAddLocalAction.php
deleted file mode 100644
index eb1381395f869e020c8c3bc06e71ae96f15a5516..0000000000000000000000000000000000000000
--- a/core/modules/image/lib/Drupal/image/Plugin/Menu/LocalAction/ImageStyleAddLocalAction.php
+++ /dev/null
@@ -1,24 +0,0 @@
-<?php
-
-/**
- * @file
- * Contains \Drupal\image\Plugin\Menu\ImageStyleAddLocalAction.
- */
-
-namespace Drupal\image\Plugin\Menu\LocalAction;
-
-use Drupal\Core\Annotation\Translation;
-use Drupal\Core\Menu\LocalActionBase;
-use Drupal\Core\Annotation\Menu\LocalAction;
-
-/**
- * @LocalAction(
- *   id = "image_style_add_action",
- *   route_name = "image.style_add",
- *   title = @Translation("Add image style"),
- *   appears_on = {"image.style_list"}
- * )
- */
-class ImageStyleAddLocalAction extends LocalActionBase {
-
-}
diff --git a/core/modules/menu/menu.local_actions.yml b/core/modules/menu/menu.local_actions.yml
new file mode 100644
index 0000000000000000000000000000000000000000..a0016bc35bf1429785e89fe7a388c49cb9d5492e
--- /dev/null
+++ b/core/modules/menu/menu.local_actions.yml
@@ -0,0 +1,5 @@
+menu_link_add:
+  route_name: menu.link_add
+  title: 'Add link'
+  appears_on:
+    - menu.menu_edit
diff --git a/core/modules/menu/menu.module b/core/modules/menu/menu.module
index 70cf6e61b84caa40f3f943def1763e41dd2d517f..5f20c48dccf4ed2ca47dae4a2247a74478448af9 100644
--- a/core/modules/menu/menu.module
+++ b/core/modules/menu/menu.module
@@ -91,11 +91,6 @@ function menu_menu() {
     'title callback' => 'entity_page_label',
     'title arguments' => array(4),
   );
-  $items['admin/structure/menu/manage/%menu/add'] = array(
-    'title' => 'Add link',
-    'route_name' => 'menu.link_add',
-    'type' => MENU_LOCAL_ACTION,
-  );
   $items['admin/structure/menu/manage/%menu/edit'] = array(
     'title' => 'Edit menu',
     'type' => MENU_DEFAULT_LOCAL_TASK,
diff --git a/core/modules/shortcut/lib/Drupal/shortcut/Plugin/Menu/LocalAction/AddShortcutSetLocalAction.php b/core/modules/shortcut/lib/Drupal/shortcut/Plugin/Menu/LocalAction/AddShortcutSetLocalAction.php
deleted file mode 100644
index 8a01f7bac9ead6647f1fec3be45c6e4bfb677800..0000000000000000000000000000000000000000
--- a/core/modules/shortcut/lib/Drupal/shortcut/Plugin/Menu/LocalAction/AddShortcutSetLocalAction.php
+++ /dev/null
@@ -1,24 +0,0 @@
-<?php
-
-/**
- * @file
- * Contains \Drupal\shortcut\Plugin\Menu\AddShortcutSetLocalAction.
- */
-
-namespace Drupal\shortcut\Plugin\Menu\LocalAction;
-
-use Drupal\Core\Annotation\Translation;
-use Drupal\Core\Annotation\Menu\LocalAction;
-use Drupal\Core\Menu\LocalActionBase;
-
-/**
- * @LocalAction(
- *   id = "shortcut_set_add_local_action",
- *   route_name = "shortcut.set_add",
- *   title = @Translation("Add shortcut set"),
- *   appears_on = {"shortcut.set_admin"}
- * )
- */
-class AddShortcutSetLocalAction extends LocalActionBase {
-
-}
diff --git a/core/modules/shortcut/shortcut.local_actions.yml b/core/modules/shortcut/shortcut.local_actions.yml
new file mode 100644
index 0000000000000000000000000000000000000000..2067f7de4550380459ae2c49d09da75bd95ecf14
--- /dev/null
+++ b/core/modules/shortcut/shortcut.local_actions.yml
@@ -0,0 +1,5 @@
+shortcut_set_add_local_action:
+  route_name: shortcut.set_add
+  title: 'Add shortcut set'
+  appears_on:
+    - shortcut.set_admin
diff --git a/core/modules/shortcut/shortcut.module b/core/modules/shortcut/shortcut.module
index 8ca0472d4689c715ad42ff5ddb604127b058ae6c..8ea2fa1bff9baa649a8c0e40aa644a9d528b3114 100644
--- a/core/modules/shortcut/shortcut.module
+++ b/core/modules/shortcut/shortcut.module
@@ -90,11 +90,6 @@ function shortcut_menu() {
     'description' => 'Add and modify shortcut sets.',
     'route_name' => 'shortcut.set_admin',
   );
-  $items['admin/config/user-interface/shortcut/add-set'] = array(
-    'title' => 'Add shortcut set',
-    'route_name' => 'shortcut.set_add',
-    'type' => MENU_SIBLING_LOCAL_TASK,
-  );
   $items['admin/config/user-interface/shortcut/manage/%shortcut_set'] = array(
     'title' => 'Edit shortcuts',
     'route_name' => 'shortcut.set_customize',
diff --git a/core/modules/system/lib/Drupal/system/Tests/Menu/LocalActionTest.php b/core/modules/system/lib/Drupal/system/Tests/Menu/LocalActionTest.php
index 014d58f5561f269bb67ef9ebe42e9e8e712ba945..d8804dd4887f2ee7bf12d6fcfc5c0a449546ce9e 100644
--- a/core/modules/system/lib/Drupal/system/Tests/Menu/LocalActionTest.php
+++ b/core/modules/system/lib/Drupal/system/Tests/Menu/LocalActionTest.php
@@ -39,8 +39,8 @@ public function testLocalAction() {
     $this->assertLocalAction(array(
       'menu-test-local-action/dynamic-title' => 'My dynamic-title action',
       'menu-test-local-action/hook_menu' => 'My hook_menu action',
-      'menu-test-local-action/routing' => 'My routing action',
       'menu-test-local-action/routing' => 'My YAML discovery action',
+      'menu-test-local-action/routing2' => 'Title override',
     ));
   }
 
diff --git a/core/modules/system/tests/modules/menu_test/lib/Drupal/menu_test/Plugin/Menu/LocalAction/MenuTestLocalAction.php b/core/modules/system/tests/modules/menu_test/lib/Drupal/menu_test/Plugin/Menu/LocalAction/MenuTestLocalAction.php
deleted file mode 100644
index fb33b7e3af1bab48d095bd41d5f1f4744212da5d..0000000000000000000000000000000000000000
--- a/core/modules/system/tests/modules/menu_test/lib/Drupal/menu_test/Plugin/Menu/LocalAction/MenuTestLocalAction.php
+++ /dev/null
@@ -1,24 +0,0 @@
-<?php
-
-/**
- * @file
- * Contains \Drupal\menu_test\Plugin\Menu\MenuTestLocalAction.
- */
-
-namespace Drupal\menu_test\Plugin\Menu\LocalAction;
-
-use Drupal\Core\Annotation\Translation;
-use Drupal\Core\Menu\LocalActionBase;
-use Drupal\Core\Annotation\Menu\LocalAction;
-
-/**
- * @LocalAction(
- *   id = "menu_test_local_action3",
- *   route_name = "menu_test.local_action3",
- *   title = @Translation("My routing action"),
- *   appears_on = {"menu_test.local_action1"}
- * )
- */
-class MenuTestLocalAction extends LocalActionBase {
-
-}
diff --git a/core/modules/system/tests/modules/menu_test/lib/Drupal/menu_test/Plugin/Menu/LocalAction/TestLocalAction.php b/core/modules/system/tests/modules/menu_test/lib/Drupal/menu_test/Plugin/Menu/LocalAction/TestLocalAction.php
new file mode 100644
index 0000000000000000000000000000000000000000..3811954ebec1e7feccba04ae848f8c23755265c8
--- /dev/null
+++ b/core/modules/system/tests/modules/menu_test/lib/Drupal/menu_test/Plugin/Menu/LocalAction/TestLocalAction.php
@@ -0,0 +1,24 @@
+<?php
+
+/**
+ * @file
+ * Contains \Drupal\menu_test\Plugin\Menu\LocalAction\TestLocalAction.
+ */
+
+namespace Drupal\menu_test\Plugin\Menu\LocalAction;
+
+use Drupal\Core\Menu\LocalActionDefault;
+
+/**
+ * Defines a test local action plugin class.
+ */
+class TestLocalAction extends LocalActionDefault {
+
+  /**
+   * {@inheritdoc}
+   */
+  public function getTitle() {
+    return 'Title override';
+  }
+
+}
diff --git a/core/modules/system/tests/modules/menu_test/menu_test.local_actions.yml b/core/modules/system/tests/modules/menu_test/menu_test.local_actions.yml
index 4f5bf55f6c660eb01c4b37734299bd24368cad0e..76bde782962773576e80c8c86bed04fe7e1885fe 100644
--- a/core/modules/system/tests/modules/menu_test/menu_test.local_actions.yml
+++ b/core/modules/system/tests/modules/menu_test/menu_test.local_actions.yml
@@ -3,3 +3,11 @@ menu_test_local_action4:
   title: 'My YAML discovery action'
   appears_on:
     - menu_test.local_action1
+
+menu_test.local_action5:
+  route_name: menu_test.local_action5
+  class: '\Drupal\menu_test\Plugin\Menu\LocalAction\TestLocalAction'
+  weight: 10
+  appears_on:
+    - menu_test.local_action1
+
diff --git a/core/modules/system/tests/modules/menu_test/menu_test.routing.yml b/core/modules/system/tests/modules/menu_test/menu_test.routing.yml
index 708816a20eb1e5da3a21cac2e3a59bed3a0aed87..fd902531c890bc26de43e1b523eecd2c1a859d14 100644
--- a/core/modules/system/tests/modules/menu_test/menu_test.routing.yml
+++ b/core/modules/system/tests/modules/menu_test/menu_test.routing.yml
@@ -61,6 +61,13 @@ menu_test.local_action4:
   requirements:
     _access: 'TRUE'
 
+menu_test.local_action5:
+  path: '/menu-test-local-action/routing2'
+  defaults:
+    _content: '\Drupal\menu_test\TestControllers::test2'
+  requirements:
+    _access: 'TRUE'
+
 menu_test.local_task_test_tasks:
   path: '/menu-local-task-test/tasks'
   defaults:
diff --git a/core/modules/system/tests/modules/router_test/router_test.routing.yml b/core/modules/system/tests/modules/router_test/router_test.routing.yml
index 0aab01a57867de34e19aac7dd592edd04fbcb64e..3217c98b5d3b7492a8e7c118b175cd1c0f4e02b0 100644
--- a/core/modules/system/tests/modules/router_test/router_test.routing.yml
+++ b/core/modules/system/tests/modules/router_test/router_test.routing.yml
@@ -48,6 +48,8 @@ router_test.8:
 
 router_test.9:
   path: '/router_test/test9'
+  options:
+    _access_mode: 'ANY'
   defaults:
     _controller: '\Drupal\router_test\TestControllers::test8'
   requirements:
diff --git a/core/modules/views_ui/views_ui.module b/core/modules/views_ui/views_ui.module
index a884a791fc44b8062f3ef9f1a7320bc8de18229e..0508ff965e4be332b3696f554f1cd83a2b003bfa 100644
--- a/core/modules/views_ui/views_ui.module
+++ b/core/modules/views_ui/views_ui.module
@@ -25,11 +25,6 @@ function views_ui_menu() {
     'route_name' => 'views_ui.list',
   );
 
-  $items['admin/structure/views/add'] = array(
-    'route_name' => 'views_ui.add',
-    'type' => MENU_SIBLING_LOCAL_TASK,
-  );
-
   // @todo - remove all items of type MENU_VISIBLE_IN_BREADCRUMB
   // when there is a route-aware breadcrumb builder.
   $items['admin/structure/views/settings'] = array(
diff --git a/core/lib/Drupal/Core/Menu/LocalActionManager.php b/core/lib/Drupal/Core/Menu/LocalActionManager.php
index 39ea454688fbaa88bc708fe6b5a5ee10e83e7d9d..a6c78b062f7fa9e4996f0de780cc3b77bf086596 100644
--- a/core/lib/Drupal/Core/Menu/LocalActionManager.php
+++ b/core/lib/Drupal/Core/Menu/LocalActionManager.php
@@ -7,11 +7,17 @@
 
 namespace Drupal\Core\Menu;
 
+use Drupal\Core\Access\AccessManager;
 use Drupal\Core\Cache\CacheBackendInterface;
 use Drupal\Core\Extension\ModuleHandlerInterface;
 use Drupal\Core\Language\LanguageManager;
 use Drupal\Core\Menu\LocalActionInterface;
 use Drupal\Core\Plugin\DefaultPluginManager;
+use Drupal\Component\Plugin\Discovery\ProcessDecorator;
+use Drupal\Core\Plugin\Discovery\ContainerDerivativeDiscoveryDecorator;
+use Drupal\Core\Plugin\Discovery\YamlDiscovery;
+use Drupal\Core\Plugin\Factory\ContainerFactory;
+use Drupal\Core\Routing\RouteProviderInterface;
 use Symfony\Component\HttpFoundation\Request;
 use Symfony\Component\HttpKernel\Controller\ControllerResolverInterface;
 
@@ -25,6 +31,30 @@
  */
 class LocalActionManager extends DefaultPluginManager {
 
+  /**
+   * Provides some default values for all local action plugins.
+   *
+   * @var array
+   */
+  protected $defaults = array(
+    // The plugin id. Set by the plugin system based on the top-level YAML key.
+    'id' => NULL,
+    // The static title for the local action.
+    'title' => '',
+    // The weight of the local action.
+    'weight' => NULL,
+    // (Required) the route name used to generate a link.
+    'route_name' => NULL,
+    // Default route parameters for generating links.
+    'route_parameters' => array(),
+    // Associative array of link options.
+    'options' => array(),
+    // The route names where this local action appears.
+    'appears_on' => array(),
+    // Default class for local action implementations.
+    'class' => 'Drupal\Core\Menu\LocalActionDefault',
+  );
+
   /**
    * A controller resolver object.
    *
@@ -40,6 +70,20 @@ class LocalActionManager extends DefaultPluginManager {
   protected $request;
 
   /**
+   * The route provider to load routes by name.
+   *
+   * @var \Drupal\Core\Routing\RouteProviderInterface
+   */
+  protected $routeProvider;
+
+  /**
+   * The access manager.
+   *
+   * @var \Drupal\Core\Access\AccessManager
+   */
+  protected $accessManager;
+
+/**
    * The plugin instances.
    *
    * @var array
@@ -49,28 +93,35 @@ class LocalActionManager extends DefaultPluginManager {
   /**
    * Constructs a LocalActionManager object.
    *
-   * @param \Traversable $namespaces
-   *   An object that implements \Traversable which contains the root paths
-   *   keyed by the corresponding namespace to look for plugin implementations.
    * @param \Symfony\Component\HttpKernel\Controller\ControllerResolverInterface $controller_resolver
    *   An object to use in introspecting route methods.
    * @param \Symfony\Component\HttpFoundation\Request $request
    *   The request object to use for building titles and paths for plugin
    *   instances.
+   * @param \Drupal\Core\Routing\RouteProviderInterface $route_provider
+   *   The route provider.
    * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
    *   The module handler.
    * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
    *   Cache backend instance to use.
    * @param \Drupal\Core\Language\LanguageManager $language_manager
    *   The language manager.
+   * @param \Drupal\Core\Access\AccessManager $access_manager
+   *   The access manager.
    */
-  public function __construct(\Traversable $namespaces, ControllerResolverInterface $controller_resolver, Request $request, ModuleHandlerInterface $module_handler, CacheBackendInterface $cache_backend, LanguageManager $language_manager) {
-    parent::__construct('Plugin/Menu/LocalAction', $namespaces, 'Drupal\Core\Annotation\Menu\LocalAction');
-
+  public function __construct(ControllerResolverInterface $controller_resolver, Request $request, RouteProviderInterface $route_provider, ModuleHandlerInterface $module_handler, CacheBackendInterface $cache_backend, LanguageManager $language_manager, AccessManager $access_manager) {
+    // Skip calling the parent constructor, since that assumes annotation-based
+    // discovery.
+    $this->discovery = new YamlDiscovery('local_actions', $module_handler->getModuleDirectories());
+    $this->discovery = new ContainerDerivativeDiscoveryDecorator($this->discovery);
+    $this->factory = new ContainerFactory($this);
+    $this->routeProvider = $route_provider;
+    $this->accessManager = $access_manager;
     $this->controllerResolver = $controller_resolver;
     $this->request = $request;
     $this->alterInfo($module_handler, 'menu_local_actions');
-    $this->setCacheBackend($cache_backend, $language_manager, 'local_action_plugins');
+    $this->setCacheBackend($cache_backend, $language_manager, 'local_action_plugins', array('local_action' => TRUE));
+
   }
 
   /**
@@ -91,45 +142,50 @@ public function getTitle(LocalActionInterface $local_action) {
     return call_user_func_array($controller, $arguments);
   }
 
-  /**
-   * Gets the Drupal path for a local action.
-   *
-   * @param \Drupal\Core\Menu\LocalActionInterface $local_action
-   *   An object to get the path from.
-   *
-   * @return string
-   *   The path.
-   *
-   * @throws \BadMethodCallException
-   *   If the plugin does not implement the getPath() method.
-   */
-  public function getPath(LocalActionInterface $local_action) {
-    $controller = array($local_action, 'getPath');
-    $arguments = $this->controllerResolver->getArguments($this->request, $controller);
-    return call_user_func_array($controller, $arguments);
-  }
-
   /**
    * Finds all local actions that appear on a named route.
    *
    * @param string $route_name
    *   The route for which to find local actions.
    *
-   * @return \Drupal\Core\Menu\LocalActionInterface[]
-   *   An array of LocalActionInterface objects that appear on the route path.
+   * @return array
+   *   An array of link render arrays.
    */
-  public function getActionsForRoute($route_name) {
-    if (!isset($this->instances[$route_name])) {
-      $this->instances[$route_name] = array();
+  public function getActionsForRoute($route_appears) {
+    if (!isset($this->instances[$route_appears])) {
+      $route_names = array();
+      $this->instances[$route_appears] = array();
       // @todo - optimize this lookup by compiling or caching.
       foreach ($this->getDefinitions() as $plugin_id => $action_info) {
-        if (in_array($route_name, $action_info['appears_on'])) {
+        if (in_array($route_appears, $action_info['appears_on'])) {
           $plugin = $this->createInstance($plugin_id);
-          $this->instances[$route_name][$plugin_id] = $plugin;
+          $route_names[] = $plugin->getRouteName();
+          $this->instances[$route_appears][$plugin_id] = $plugin;
         }
       }
+      // Pre-fetch all the action route objects. This reduces the number of SQL
+      // queries that would otherwise be triggered by the access manager.
+      if (!empty($route_names)) {
+        $this->routeProvider->getRoutesByNames($route_names);
+      }
+    }
+    $links = array();
+    foreach ($this->instances[$route_appears] as $plugin) {
+      $route_name = $plugin->getRouteName();
+      $route_parameters = $plugin->getRouteParameters($this->request);
+      $links[$route_name] = array(
+        '#theme' => 'menu_local_action',
+        '#link' => array(
+          'title' => $this->getTitle($plugin),
+          'route_name' => $route_name,
+          'route_parameters' => $route_parameters,
+          'localized_options' => $plugin->getOptions($this->request),
+        ),
+        '#access' => $this->accessManager->checkNamedRoute($route_name, $route_parameters),
+        '#weight' => $plugin->getWeight(),
+      );
     }
-    return $this->instances[$route_name];
+    return $links;
   }
 
 }
@@ -91,45 +142,50 @@ public function getTitle(LocalActionInterface $local_action) {
    * Finds all local actions that appear on a named route.
    *
-   * @param string $route_name
-   *   The route for which to find local actions.
+   * @param string $route_appears
+   *   The route name for which to find local actions.
    *